# -*- coding: utf-8 -*-
"""
Porpose: audiomass __init__ file that holds package information.
Writer: Gianluca Pernigoto <jeanlucperni@gmail.com>
Copyright: (c) 2015/2020 Gianluca Pernigoto <jeanlucperni@gmail.com>
license: GPL3
Rev: July.25.2020, Dec 08 2021
Code checker: flake8, pylint
"""

__author__ = ('Gianluca Pernigotto', '(aka jeanslack)')
__mail__ = '<jeanlucperni@gmail.com>'
__copyright__ = '© 2013-2021'
__version__ = '0.9.5'
__release__ = 'December 16 2021'
__rls_name__ = "Audiomass"
__prg_name__ = "audiomass"
__url__ = "https://github.com/jeanslack/audiomass"
__short_descript__ = 'Wrapper for multiple audio conversion libraries. '
__long_descript__ = """
**Audiomass** is a command line audio wrapper of the Flac, Lame, Vorbis-tools,
Monkey's Audio, Shntool and FFmpeg libraries. It supports conversions of
different audio formats at a time and the ability to convert even groups of
files in a directory, saving the output in a specific folder.
"""

__try__ = "type 'audiomass -h' for help or, more detailed, run 'man audiomass'"
__license__ = (f"""
    Copyright {__copyright__} - {__author__[0]} {__author__[1]}
    Mail: {__mail__}

    Audiomass is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or (at
    your option) any later version.

    Audiomass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with audiomass; if not, see <http://www.gnu.org/licenses/>
    or write to the Free Software Foundation, Inc., 51 Franklin St,
    Fifth Floor, Boston, MA  02110-1301 USA
    ==============================================================
    """)

__short_license__ = "'Gnu GPL3 (Gnu Public License)"
